/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

/*
 * This file is auto-generated by Qpid Gentools v.0.1 - do not modify.
 * Supported AMQP versions:
 *   0-9
 */
 
#ifndef qpid_framing_AMQP_ServerOperations__
#define qpid_framing_AMQP_ServerOperations__

#include "qpid/framing/ProtocolVersion.h"

namespace qpid {
namespace framing {

class MethodContext;

class AMQP_ServerOperations
{
protected:
    ProtocolVersion version;

public:
    virtual ~AMQP_ServerOperations() {}

    virtual ProtocolVersion getVersion() const = 0;
    
    // Include framing constant declarations
    #include "AMQP_Constants.h"

    // Inner classes

    // ==================== class AccessHandler ====================
    class AccessHandler
    {
        // Constructors and destructors
    public:
        AccessHandler(){};
        virtual ~AccessHandler() {}

        // Protocol methods

        virtual void request(const MethodContext& context,
                            const string& realm,
                            bool exclusive,
                            bool passive,
                            bool active,
                            bool write,
                            bool read ) = 0;
    }; // class AccessHandler

    // ==================== class BasicHandler ====================
    class BasicHandler
    {
        // Constructors and destructors
    public:
        BasicHandler(){};
        virtual ~BasicHandler() {}

        // Protocol methods

        virtual void ack(const MethodContext& context,
                            u_int64_t deliveryTag,
                            bool multiple ) = 0;

        virtual void cancel(const MethodContext& context,
                            const string& consumerTag,
                            bool nowait ) = 0;

        virtual void consume(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            const string& consumerTag,
                            bool noLocal,
                            bool noAck,
                            bool exclusive,
                            bool nowait,
                            const FieldTable& filter ) = 0;

        virtual void get(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            bool noAck ) = 0;

        virtual void publish(const MethodContext& context,
                            u_int16_t ticket,
                            const string& exchange,
                            const string& routingKey,
                            bool mandatory,
                            bool immediate ) = 0;

        virtual void qos(const MethodContext& context,
                            u_int32_t prefetchSize,
                            u_int16_t prefetchCount,
                            bool global ) = 0;

        virtual void recover(const MethodContext& context,
                            bool requeue ) = 0;

        virtual void reject(const MethodContext& context,
                            u_int64_t deliveryTag,
                            bool requeue ) = 0;
    }; // class BasicHandler

    // ==================== class ChannelHandler ====================
    class ChannelHandler
    {
        // Constructors and destructors
    public:
        ChannelHandler(){};
        virtual ~ChannelHandler() {}

        // Protocol methods

        virtual void close(const MethodContext& context,
                            u_int16_t replyCode,
                            const string& replyText,
                            u_int16_t classId,
                            u_int16_t methodId ) = 0;

        virtual void closeOk(const MethodContext& context ) = 0;

        virtual void flow(const MethodContext& context,
                            bool active ) = 0;

        virtual void flowOk(const MethodContext& context,
                            bool active ) = 0;

        virtual void ok(const MethodContext& context ) = 0;

        virtual void open(const MethodContext& context,
                            const string& outOfBand ) = 0;

        virtual void ping(const MethodContext& context ) = 0;

        virtual void pong(const MethodContext& context ) = 0;

        virtual void resume(const MethodContext& context,
                            const string& channelId ) = 0;
    }; // class ChannelHandler

    // ==================== class ConnectionHandler ====================
    class ConnectionHandler
    {
        // Constructors and destructors
    public:
        ConnectionHandler(){};
        virtual ~ConnectionHandler() {}

        // Protocol methods

        virtual void close(const MethodContext& context,
                            u_int16_t replyCode,
                            const string& replyText,
                            u_int16_t classId,
                            u_int16_t methodId ) = 0;

        virtual void closeOk(const MethodContext& context ) = 0;

        virtual void open(const MethodContext& context,
                            const string& virtualHost,
                            const string& capabilities,
                            bool insist ) = 0;

        virtual void secureOk(const MethodContext& context,
                            const string& response ) = 0;

        virtual void startOk(const MethodContext& context,
                            const FieldTable& clientProperties,
                            const string& mechanism,
                            const string& response,
                            const string& locale ) = 0;

        virtual void tuneOk(const MethodContext& context,
                            u_int16_t channelMax,
                            u_int32_t frameMax,
                            u_int16_t heartbeat ) = 0;
    }; // class ConnectionHandler

    // ==================== class DtxHandler ====================
    class DtxHandler
    {
        // Constructors and destructors
    public:
        DtxHandler(){};
        virtual ~DtxHandler() {}

        // Protocol methods

        virtual void select(const MethodContext& context ) = 0;

        virtual void start(const MethodContext& context,
                            const string& dtxIdentifier ) = 0;
    }; // class DtxHandler

    // ==================== class ExchangeHandler ====================
    class ExchangeHandler
    {
        // Constructors and destructors
    public:
        ExchangeHandler(){};
        virtual ~ExchangeHandler() {}

        // Protocol methods

        virtual void declare(const MethodContext& context,
                            u_int16_t ticket,
                            const string& exchange,
                            const string& type,
                            bool passive,
                            bool durable,
                            bool autoDelete,
                            bool internal,
                            bool nowait,
                            const FieldTable& arguments ) = 0;

        virtual void delete_(const MethodContext& context,
                            u_int16_t ticket,
                            const string& exchange,
                            bool ifUnused,
                            bool nowait ) = 0;
    }; // class ExchangeHandler

    // ==================== class FileHandler ====================
    class FileHandler
    {
        // Constructors and destructors
    public:
        FileHandler(){};
        virtual ~FileHandler() {}

        // Protocol methods

        virtual void ack(const MethodContext& context,
                            u_int64_t deliveryTag,
                            bool multiple ) = 0;

        virtual void cancel(const MethodContext& context,
                            const string& consumerTag,
                            bool nowait ) = 0;

        virtual void consume(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            const string& consumerTag,
                            bool noLocal,
                            bool noAck,
                            bool exclusive,
                            bool nowait,
                            const FieldTable& filter ) = 0;

        virtual void open(const MethodContext& context,
                            const string& identifier,
                            u_int64_t contentSize ) = 0;

        virtual void openOk(const MethodContext& context,
                            u_int64_t stagedSize ) = 0;

        virtual void publish(const MethodContext& context,
                            u_int16_t ticket,
                            const string& exchange,
                            const string& routingKey,
                            bool mandatory,
                            bool immediate,
                            const string& identifier ) = 0;

        virtual void qos(const MethodContext& context,
                            u_int32_t prefetchSize,
                            u_int16_t prefetchCount,
                            bool global ) = 0;

        virtual void reject(const MethodContext& context,
                            u_int64_t deliveryTag,
                            bool requeue ) = 0;

        virtual void stage(const MethodContext& context ) = 0;
    }; // class FileHandler

    // ==================== class MessageHandler ====================
    class MessageHandler
    {
        // Constructors and destructors
    public:
        MessageHandler(){};
        virtual ~MessageHandler() {}

        // Protocol methods

        virtual void append(const MethodContext& context,
                            const string& reference,
                            const string& bytes ) = 0;

        virtual void cancel(const MethodContext& context,
                            const string& destination ) = 0;

        virtual void checkpoint(const MethodContext& context,
                            const string& reference,
                            const string& identifier ) = 0;

        virtual void close(const MethodContext& context,
                            const string& reference ) = 0;

        virtual void consume(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            const string& destination,
                            bool noLocal,
                            bool noAck,
                            bool exclusive,
                            const FieldTable& filter ) = 0;

        virtual void empty(const MethodContext& context ) = 0;

        virtual void get(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            const string& destination,
                            bool noAck ) = 0;

        virtual void offset(const MethodContext& context,
                            u_int64_t value ) = 0;

        virtual void ok(const MethodContext& context ) = 0;

        virtual void open(const MethodContext& context,
                            const string& reference ) = 0;

        virtual void qos(const MethodContext& context,
                            u_int32_t prefetchSize,
                            u_int16_t prefetchCount,
                            bool global ) = 0;

        virtual void recover(const MethodContext& context,
                            bool requeue ) = 0;

        virtual void reject(const MethodContext& context,
                            u_int16_t code,
                            const string& text ) = 0;

        virtual void resume(const MethodContext& context,
                            const string& reference,
                            const string& identifier ) = 0;

        virtual void transfer(const MethodContext& context,
                            u_int16_t ticket,
                            const string& destination,
                            bool redelivered,
                            bool immediate,
                            u_int64_t ttl,
                            u_int8_t priority,
                            u_int64_t timestamp,
                            u_int8_t deliveryMode,
                            u_int64_t expiration,
                            const string& exchange,
                            const string& routingKey,
                            const string& messageId,
                            const string& correlationId,
                            const string& replyTo,
                            const string& contentType,
                            const string& contentEncoding,
                            const string& userId,
                            const string& appId,
                            const string& transactionId,
                            const string& securityToken,
                            const FieldTable& applicationHeaders,
                            const Content& body,
                            bool mandatory ) = 0;
    }; // class MessageHandler

    // ==================== class QueueHandler ====================
    class QueueHandler
    {
        // Constructors and destructors
    public:
        QueueHandler(){};
        virtual ~QueueHandler() {}

        // Protocol methods

        virtual void bind(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            const string& exchange,
                            const string& routingKey,
                            bool nowait,
                            const FieldTable& arguments ) = 0;

        virtual void declare(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            bool passive,
                            bool durable,
                            bool exclusive,
                            bool autoDelete,
                            bool nowait,
                            const FieldTable& arguments ) = 0;

        virtual void delete_(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            bool ifUnused,
                            bool ifEmpty,
                            bool nowait ) = 0;

        virtual void purge(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            bool nowait ) = 0;

        virtual void unbind(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            const string& exchange,
                            const string& routingKey,
                            const FieldTable& arguments ) = 0;
    }; // class QueueHandler

    // ==================== class StreamHandler ====================
    class StreamHandler
    {
        // Constructors and destructors
    public:
        StreamHandler(){};
        virtual ~StreamHandler() {}

        // Protocol methods

        virtual void cancel(const MethodContext& context,
                            const string& consumerTag,
                            bool nowait ) = 0;

        virtual void consume(const MethodContext& context,
                            u_int16_t ticket,
                            const string& queue,
                            const string& consumerTag,
                            bool noLocal,
                            bool exclusive,
                            bool nowait,
                            const FieldTable& filter ) = 0;

        virtual void publish(const MethodContext& context,
                            u_int16_t ticket,
                            const string& exchange,
                            const string& routingKey,
                            bool mandatory,
                            bool immediate ) = 0;

        virtual void qos(const MethodContext& context,
                            u_int32_t prefetchSize,
                            u_int16_t prefetchCount,
                            u_int32_t consumeRate,
                            bool global ) = 0;
    }; // class StreamHandler

    // ==================== class TunnelHandler ====================
    class TunnelHandler
    {
        // Constructors and destructors
    public:
        TunnelHandler(){};
        virtual ~TunnelHandler() {}

        // Protocol methods

        virtual void request(const MethodContext& context,
                            const FieldTable& metaData ) = 0;
    }; // class TunnelHandler

    // ==================== class TxHandler ====================
    class TxHandler
    {
        // Constructors and destructors
    public:
        TxHandler(){};
        virtual ~TxHandler() {}

        // Protocol methods

        virtual void commit(const MethodContext& context ) = 0;

        virtual void rollback(const MethodContext& context ) = 0;

        virtual void select(const MethodContext& context ) = 0;
    }; // class TxHandler

    // Method handler get methods

    virtual AMQP_ServerOperations::AccessHandler* getAccessHandler() = 0;
    virtual AMQP_ServerOperations::BasicHandler* getBasicHandler() = 0;
    virtual AMQP_ServerOperations::ChannelHandler* getChannelHandler() = 0;
    virtual AMQP_ServerOperations::ConnectionHandler* getConnectionHandler() = 0;
    virtual AMQP_ServerOperations::DtxHandler* getDtxHandler() = 0;
    virtual AMQP_ServerOperations::ExchangeHandler* getExchangeHandler() = 0;
    virtual AMQP_ServerOperations::FileHandler* getFileHandler() = 0;
    virtual AMQP_ServerOperations::MessageHandler* getMessageHandler() = 0;
    virtual AMQP_ServerOperations::QueueHandler* getQueueHandler() = 0;
    virtual AMQP_ServerOperations::StreamHandler* getStreamHandler() = 0;
    virtual AMQP_ServerOperations::TunnelHandler* getTunnelHandler() = 0;
    virtual AMQP_ServerOperations::TxHandler* getTxHandler() = 0;

}; /* class AMQP_ServerOperations */

} /* namespace framing */
} /* namespace qpid */

#endif
